<?php

$lang['profiler_database']		= 'БД';
$lang['profiler_controller_info'] = 'Класс/Метод';
$lang['profiler_benchmarks']	= 'Критерии';
$lang['profiler_queries']		= 'Запросы';
$lang['profiler_get_data']		= 'Данные GET';
$lang['profiler_post_data']		= 'Данные POST';
$lang['profiler_uri_string']	= 'URI STRING';
$lang['profiler_memory_usage']	= 'Использование памяти';
$lang['profiler_config']		= 'Переменные конфигурации';
$lang['profiler_session_data']	= 'SESSION DATA';
$lang['profiler_headers']		= 'Заголовки HTTP';
$lang['profiler_no_db']			= 'Драйвер БД не загружен';
$lang['profiler_no_queries']	= 'Нет запросов в БД';
$lang['profiler_no_post']		= 'Нет данных POST';
$lang['profiler_no_get']		= 'Нет данных GET';
$lang['profiler_no_uri']		= 'Нет данных URI';
$lang['profiler_no_memory']		= 'Данные по использованию памяти недоступны';
$lang['profiler_no_profiles']	= 'Нет данных для профилирования - все секции профайлера выключены.';
$lang['profiler_section_hide']	= 'Hide';
$lang['profiler_section_show']	= 'Show';

/* End of file profiler_lang.php */
/* Location: ./system/language/english/profiler_lang.php */
