<?php

$lang['upload_userfile_not_set'] = "Не удается найти переменную userfile в POST.";
$lang['upload_file_exceeds_limit'] = "Размер загружаемого файла превышает максимально допустимый размер, указанный в конфигурации вашего сервера.";
$lang['upload_file_exceeds_form_limit'] = "Размер загружаемого файла превышает разрешенный в форме.";
$lang['upload_file_partial'] = "Файл загружен частично.";
$lang['upload_no_temp_directory'] = "Директория для хранения временных файлов отсутствует.";
$lang['upload_unable_to_write_file'] = "Файл не может быть записан на диск.";
$lang['upload_stopped_by_extension'] = "Загрузка файла остановлена по расширению.";
$lang['upload_no_file_selected'] = "Вы не выбрали файл для загрузки.";
$lang['upload_invalid_filetype'] = "Загрузка этого типа файлов не допускается.";
$lang['upload_invalid_filesize'] = "Слишком большой файл.";
$lang['upload_invalid_dimensions'] = "Слишком большое изображение.";
$lang['upload_destination_error'] = "Возникла проблема при перемещении загруженного файла.";
$lang['upload_no_filepath'] = "Некорректный путь для загрузки.";
$lang['upload_no_file_types'] = "Вы не указали разрешенные типы файлов.";
$lang['upload_bad_filename'] = "Этот файл уже присутствует на сервере.";
$lang['upload_not_writable'] = "Директория для загружаемых файлов не доступна.";


/* End of file upload_lang.php */
/* Location: ./system/language/english/upload_lang.php */
