<?php

$lang['ftp_no_connection']			= "Не удалось найти действительный идентификатор соединения. Пожалуйста убедитесь, что соединение установлено, прежде чем выполнять любые процедуры с файлами.";
$lang['ftp_unable_to_connect']		= "Невозможно установить соединение с FTP-сервером, используя имеющиеся настройки.";
$lang['ftp_unable_to_login']		= "Невозможно войти на FTP-сервер.  Пожалуйста, проверьте имя пользователя и пароль.";
$lang['ftp_unable_to_makdir']		= "Невозможно создать директорию.";
$lang['ftp_unable_to_changedir']	= "Невозможно изменить директорию.";
$lang['ftp_unable_to_chmod']		= "Невозможно изменить права.  Пожалуйста, проверьте путь.";
$lang['ftp_unable_to_upload']		= "Невозможно загрузить требуемый файл. Пожалуйста, проверьте путь.";
$lang['ftp_unable_to_download']		= "Невозможно скачать требуемый файл. Пожалуйста, проверьте путь.";
$lang['ftp_no_source_file']		= "Невозможно найти исходный файл. Пожалуйста, проверьте путь.";
$lang['ftp_unable_to_rename']		= "Невозможно переименовать файл.";
$lang['ftp_unable_to_delete']		= "Невозможно удалить файл.";
$lang['ftp_unable_to_move']			= "Невозможно переместить файл. Пожалуйста, удостоверьтесь в существовании целевой директории.";


/* End of file ftp_lang.php */
/* Location: ./system/language/english/ftp_lang.php */
