<?php

$lang['cal_su']			= "ВС";
$lang['cal_mo']			= "ПН";
$lang['cal_tu']			= "ВТ";
$lang['cal_we']			= "СР";
$lang['cal_th']			= "ЧТ";
$lang['cal_fr']			= "ПТ";
$lang['cal_sa']			= "СБ";
$lang['cal_sun']		= "Вос";
$lang['cal_mon']		= "Пон";
$lang['cal_tue']		= "Втр";
$lang['cal_wed']		= "Срд";
$lang['cal_thu']		= "Чтв";
$lang['cal_fri']		= "Пят";
$lang['cal_sat']		= "Суб";
$lang['cal_sunday']		= "Воскресенье";
$lang['cal_monday']		= "Понедельник";
$lang['cal_tuesday']	= "Вторник";
$lang['cal_wednesday']	= "Среда";
$lang['cal_thursday']	= "Четверг";
$lang['cal_friday']		= "Пятница";
$lang['cal_saturday']	= "Суббота";
$lang['cal_jan']		= "Янв";
$lang['cal_feb']		= "Фев";
$lang['cal_mar']		= "Мар";
$lang['cal_apr']		= "Апр";
$lang['cal_may']		= "Май";
$lang['cal_jun']		= "Июн";
$lang['cal_jul']		= "Июл";
$lang['cal_aug']		= "Авг";
$lang['cal_sep']		= "Сен";
$lang['cal_oct']		= "Окт";
$lang['cal_nov']		= "Ноя";
$lang['cal_dec']		= "Дек";
$lang['cal_january']	= "Январь";
$lang['cal_february']	= "Февраль";
$lang['cal_march']		= "Март";
$lang['cal_april']		= "Апрель";
$lang['cal_mayl']		= "Май";
$lang['cal_june']		= "Июнь";
$lang['cal_july']		= "Июль";
$lang['cal_august']		= "Август";
$lang['cal_september']	= "Сентябрь";
$lang['cal_october']	= "Октябрь";
$lang['cal_november']	= "Ноябрь";
$lang['cal_december']	= "Декабрь";


/* End of file calendar_lang.php */
/* Location: ./system/language/english/calendar_lang.php */
