<?php

$lang['imglib_source_image_required'] = "Укажите исходное изображение.";
$lang['imglib_gd_required'] = "Для этой функции требуется библиотека GD.";
$lang['imglib_gd_required_for_props'] = "Требуется библиотека GD для того, чтобы получить свойства изображения.";
$lang['imglib_unsupported_imagecreate'] = "Библиотека GD, установленная на вашим сервере, не поддерживает этот тип изображений.";
$lang['imglib_gif_not_supported'] = "GIF не поддерживаются по лицензионным ограничениям. Используйте JPG или PNG.";
$lang['imglib_jpg_not_supported'] = "JPG не поддерживается.";
$lang['imglib_png_not_supported'] = "PNG не поддерживается.";
$lang['imglib_jpg_or_png_required'] = "Изменение размеров изображений, доступное на вашем сервере, возможно только с форматами JPEG и PNG.";
$lang['imglib_copy_error'] = "Ошибка при копировании файла. Пожалуйста, проверьте доступность директории для записи.";
$lang['imglib_rotate_unsupported'] = "Вращение изображений не поддерживается на вашем сервере.";
$lang['imglib_libpath_invalid'] = "Путь к библиотеке для обработки изображений некорректный.";
$lang['imglib_image_process_failed'] = "Обработка изображения не удалась. Пожалуйста, проверьте настройки и доступность директории для сохранения изображения.";
$lang['imglib_rotation_angle_required'] = "Для вращения изображения требуется указать угол.";
$lang['imglib_writing_failed_gif'] = "GIF";
$lang['imglib_invalid_path'] = "Некорректный путь к изображению.";
$lang['imglib_copy_failed'] = "Копирование изображения не удалось.";
$lang['imglib_missing_font'] = "Невозможно найти шрифт.";
$lang['imglib_save_failed'] = "Невозможно сохранить изображение. Пожалуйста убедитесь в том, что директория для сохранения доступна.";


/* End of file imglib_lang.php */
/* Location: ./system/language/english/imglib_lang.php */
