<?php

$lang['db_invalid_connection_str'] = 'Не представляется возможным установить соединение с БД на основе строки, которую вы предоставили.';
$lang['db_unable_to_connect'] = 'Невозможно соединиться с БД по имеющимся настройкам.';
$lang['db_unable_to_select'] = 'Невозможно выбрать БД: %s';
$lang['db_unable_to_create'] = 'Невозможно создать БД: %s';
$lang['db_invalid_query'] = 'Некорректный запрос.';
$lang['db_must_set_table'] = 'Вы должны установить таблицы БД, для того, чтобы делать запросы.';
$lang['db_must_use_set'] = 'Используйте метод "set" для обновления или записи.';
$lang['db_must_use_index'] = 'Укажите индекс для пакетного обновления.';
$lang['db_batch_missing_index'] = 'Одна или несколько строк отсутствуют в индексе для пакетного обновления.';
$lang['db_must_use_where'] = 'Не допускаются обновления без условия "where".';
$lang['db_del_must_use_where'] = 'Не допускается удаление без условий "where" или "like".';
$lang['db_field_param_missing'] = 'Для получения данных требуется имя таблицы.';
$lang['db_unsupported_function'] = 'Эта функция недоступна для этой БД.';
$lang['db_transaction_failure'] = 'Сбой транзакции. Осуществляется откат';
$lang['db_unable_to_drop'] = 'Невозможно удалить указанную БД.';
$lang['db_unsuported_feature'] = 'Эта функция недоступна для этого типа БД.';
$lang['db_unsuported_compression'] = 'Этот формат сжатия не поддерживается вашим сервером.';
$lang['db_filepath_error'] = 'Невозможно записать данные по этому пути.';
$lang['db_invalid_cache_path'] = 'Этот кеш некорректный, или недоступен для записи.';
$lang['db_table_name_required'] = 'Для этой операции требуется указать имя таблицы.';
$lang['db_column_name_required'] = 'Для этой операции требуется указать имя столбца.';
$lang['db_column_definition_required'] = 'Для этой операции требуется определение столбца.';
$lang['db_unable_to_set_charset'] = 'Невозможно установить кодировку: %s';
$lang['db_error_heading'] = 'Ошибка БД';

/* End of file db_lang.php */
/* Location: ./system/language/english/db_lang.php */
