<?php

$lang['required']			= "Требуется указать &laquo;%s&raquo;.";
$lang['isset']				= "&laquo;%s&raquo; должно содержать значение.";
$lang['valid_email']		= "&laquo;%s&raquo; должно содержать корректный адрес почты.";
$lang['valid_emails']		= "&laquo;%s&raquo; должны содержать корректные почтовые адреса.";
$lang['valid_url']			= "&laquo;%s&raquo; должно содержать правильный URL.";
$lang['valid_ip']			= "&laquo;%s&raquo; должно быть правильным  IP.";
$lang['min_length']			= "&laquo;%s&raquo; должно быть не короче %s симв.";
$lang['max_length']			= "&laquo;%s&raquo; должно быть не длиннее %s симв.";
$lang['exact_length']		= "&laquo;%s&raquo; должно быть из  %s симв.";
$lang['alpha']				= "&laquo;%s&raquo; может содержать только буквы.";
$lang['alpha_numeric']		= "&laquo;%s&raquo; может содержать только буквы и цифры.";
$lang['alpha_dash']			= "&laquo;%s&raquo; может содержать только буквы, цифры, символы подчеркивания и тире.";
$lang['numeric']			= "&laquo;%s&raquo; может содержать только числа.";
$lang['is_numeric']			= "&laquo;%s&raquo; может содержать только цифры.";
$lang['integer']			= "&laquo;%s&raquo; должно быть целым числом.";
$lang['matches']			= "&laquo;%s&raquo; должно соответствовать &laquo;%s&raquo;.";
$lang['is_natural']			= "&laquo;%s&raquo; может содержать только положительные числа.";
$lang['is_natural_no_zero']	= "&laquo;%s&raquo; может содержать только положительные числа больше ноля.";


/* End of file form_validation_lang.php */
/* Location: ./system/language/english/form_validation_lang.php */
