<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <title><?php echo $heading; ?></title>
    <link href="/assets/components/bootstrap/css/bootstrap.min.css" rel="stylesheet" type="text/css" />
    <link href="/assets/components/ciext/css/main.css" rel="stylesheet" type="text/css" />
    <link href="/assets/components/ciext/css/error404.css" rel="stylesheet" type="text/css" />
</head>
<body>
<div class="container">
    <div class="error404">
        <div class="logo"></div>
        <h5><?php echo $heading; ?></h5>
        <?php echo $message; ?>
    </div>
    <?php include APPPATH . '/errors/error_footer.php'; ?>
</div>
</body>
</html>