<?php

$lang['email_must_be_array'] = "Метод проверки адреса должен быть передан в массиве.";
$lang['email_invalid_address'] = "Некорректный адрес почты: %s";
$lang['email_attachment_missing'] = "Не удается найти вложения: %s";
$lang['email_attachment_unreadable'] = "Невозможно найти вложение: %s";
$lang['email_no_recipients'] = "Укажите получателей: To, Cc или Bcc";
$lang['email_send_failure_phpmail'] = "Невозможно отправить почту, используя PHP mail(). Возможно, ваш сервер не отконфигурирован для поддержки этого метода.";
$lang['email_send_failure_sendmail'] = "Невозможно отправить почту, используя PHP Sendmail. Возможно, ваш сервер не отконфигурирован для поддержки этого метода.";
$lang['email_send_failure_smtp'] = "Невозможно отправить почту, используя PHP SMTP. Возможно, ваш сервер не отконфигурирован для поддержки этого метода.";
$lang['email_sent'] = "Сообщение успешно отправлено по протоколу : %s";
$lang['email_no_socket'] = "Невозможно открыть сокет к Sendmail. Пожалуйста, проверьте настройки.";
$lang['email_no_hostname'] = "Вы не указали имя хоста SMTP.";
$lang['email_smtp_error'] = "Произошла ошибка SMTP: %s";
$lang['email_no_smtp_unpw'] = "Ошибка: вы должны указать имя пользователя и пароль SMTP .";
$lang['email_failed_smtp_login'] = "Отказ в отправке команды AUTH LOGIN. Ошибка: %s";
$lang['email_smtp_auth_un'] = "Отказ в авторизации имени пользователя. Ошибка: %s";
$lang['email_smtp_auth_pw'] = "Не принят пароль. Ошибка: %s";
$lang['email_smtp_data_failure'] = "Невозможно отправить данные: %s";
$lang['email_exit_status'] = "Код ошибки: %s";


/* End of file email_lang.php */
/* Location: ./system/language/english/email_lang.php */
