<?php

$lang['ut_test_name']		= 'Имя теста';
$lang['ut_test_datatype']	= 'Тип данных теста';
$lang['ut_res_datatype']	= 'Ожидаемые типы данных';
$lang['ut_result']			= 'Результат';
$lang['ut_undefined']		= 'Не определено имя теста';
$lang['ut_file']			= 'Имя файла';
$lang['ut_line']			= 'Номер строки';
$lang['ut_passed']			= 'Пройдено';
$lang['ut_failed']			= 'Не удалось';
$lang['ut_boolean']			= 'Булево';
$lang['ut_integer']			= 'Целое';
$lang['ut_float']			= 'Вещественное';
$lang['ut_double']			= 'Вещественное'; // can be the same as float
$lang['ut_string']			= 'Строка';
$lang['ut_array']			= 'Массив';
$lang['ut_object']			= 'Объект';
$lang['ut_resource']		= 'Ресурс';
$lang['ut_null']			= 'Null';
$lang['ut_notes']			= 'Заметки';


/* End of file unit_test_lang.php */
/* Location: ./system/language/english/unit_test_lang.php */
